#include <stdio.h>
#include <string.h>
#include "common_porting.h"
#include "cmsis_os.h"
#include "stm32f4xx_hal.h"
#include "bmx160_task.h"
#include "bmi160.h"

#if defined(USE_BMI160)


extern volatile uint8_t int1_flag;
extern volatile uint8_t int2_flag;

struct bmi160_dev bmi160dev;
struct bmi160_sensor_data accel;
struct bmi160_sensor_data gyro;
#if defined(STEP_COUNTER)
uint16_t bmi160_step_count = 0;//stores the step counter value
#endif
#if defined(FIFO_POLL) || defined(FIFO_WM_INT) || defined(STEP_COUNTER)
TimerHandle_t BMI160_Timer_Handler;
uint16_t bmi160_fifo_ready = 0;
uint8_t bmi160_step_print = 0;

/* Declare memory to store the raw FIFO buffer information */
uint8_t fifo_buff[1024];
/* Modify the FIFO buffer instance and link to the device instance */
struct bmi160_fifo_frame fifo_frame;
struct bmi160_sensor_data fifo_acc_data[100];
struct bmi160_sensor_data fifo_gyro_data[100];
uint8_t acc_frames_req = 50; 
uint8_t gyro_frames_req = 50; 
uint16_t bmi160_fifo_idx;
#endif

void print_bmi160_rslt(int8_t rslt)
{
	switch (rslt)
	{
		case BMI160_OK:

		/* Do nothing */
		break;
		case BMI160_E_NULL_PTR:
		PDEBUG("Error [%d] : Null pointer\r\n", rslt);
		break;
		case BMI160_E_COM_FAIL:
		PDEBUG("Error [%d] : Communication failure\r\n", rslt);
		break;
		case BMI160_E_DEV_NOT_FOUND:
		PDEBUG("Error [%d] : Device not found\r\n", rslt);
		break;
		case BMI160_E_OUT_OF_RANGE:
		PDEBUG("Error [%d] : Out of range\r\n", rslt);
		break;
		case BMI160_E_INVALID_INPUT:
		PDEBUG("Error [%d] : Invalid input\r\n", rslt);
		break;
		case BMI160_E_ACCEL_ODR_BW_INVALID:
		PDEBUG("Error [%d] : BMI160_E_ACCEL_ODR_BW_INVALID\r\n", rslt);
		break;
		case BMI160_E_GYRO_ODR_BW_INVALID:
		PDEBUG("Error [%d] : BMI160_E_GYRO_ODR_BW_INVALID\r\n", rslt);
		break;
		case BMI160_E_LWP_PRE_FLTR_INT_INVALID:
		PDEBUG("Error [%d] : BMI160_E_LWP_PRE_FLTR_INT_INVALID\r\n", rslt);
		break;
		case BMI160_E_LWP_PRE_FLTR_INVALID:
		PDEBUG("Error [%d] : BMI160_E_LWP_PRE_FLTR_INVALID\r\n", rslt);
		break;
		case BMI160_E_AUX_NOT_FOUND:
		PDEBUG("Error [%d] : BMI160_E_AUX_NOT_FOUND\r\n", rslt);
		break;
		case BMI160_FOC_FAILURE:
		PDEBUG("Error [%d] : BMI160_FOC_FAILURE\r\n", rslt);
		break;
		case BMI160_READ_WRITE_LENGHT_INVALID:
		PDEBUG("Error [%d] : BMI160_READ_WRITE_LENGHT_INVALID\r\n", rslt);
		break;
		default:
		PDEBUG("Error [%d] : Unknown error code\r\n", rslt);
		break;
	}
}


int8_t Open_BMI160_ACC(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	/* Select the Output data rate, range of accelerometer sensor */
	dev->accel_cfg.odr = BMI160_ACCEL_ODR_200HZ;
	dev->accel_cfg.range = BMI160_ACCEL_RANGE_2G;
#if defined(SUPPORT_LOWPOWER)
	/* Select the power mode of accelerometer sensor */
	dev->accel_cfg.power = BMI160_ACCEL_LOWPOWER_MODE;
	dev->accel_cfg.bw = BMI160_ACCEL_BW_OSR4_AVG1;
#else
	/* Select the power mode of accelerometer sensor */
	dev->accel_cfg.power = BMI160_ACCEL_NORMAL_MODE;
	dev->accel_cfg.bw = BMI160_ACCEL_BW_NORMAL_AVG4;
#endif
	dev->gyro_cfg.power = BMI160_GYRO_SUSPEND_MODE; 


	/* Set the sensor configuration */
	rslt = bmi160_set_sens_conf(dev);
	PDEBUG("bmi160_set_sens_conf rslt=%d\r\n", rslt);
	
	return rslt;
}

int8_t Close_BMI160_ACC(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	dev->accel_cfg.power = BMI160_ACCEL_SUSPEND_MODE;
	/* Set the sensor configuration */
	rslt = bmi160_set_sens_conf(dev);
	
	return rslt;
}

int8_t Open_BMI160_GYRO(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	/* Select the Output data rate, range of Gyroscope sensor */
	dev->gyro_cfg.odr = BMI160_GYRO_ODR_200HZ;
	dev->gyro_cfg.range = BMI160_GYRO_RANGE_2000_DPS;
#if defined(SUPPORT_LOWPOWER)
	/* Select the power mode of Gyroscope sensor */
	dev->gyro_cfg.power = BMI160_GYRO_NORMAL_MODE; 
	dev->gyro_cfg.bw = BMI160_GYRO_BW_OSR4_MODE;
#else
	/* Select the power mode of Gyroscope sensor */
	dev->gyro_cfg.power = BMI160_GYRO_NORMAL_MODE; 
	dev->gyro_cfg.bw = BMI160_GYRO_BW_NORMAL_MODE;
#endif

	/* Set the sensor configuration */
	rslt = bmi160_set_sens_conf(dev);
	
	return rslt;
}

int8_t Close_BMI160_GYRO(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	dev->gyro_cfg.power = BMI160_GYRO_SUSPEND_MODE; 
	/* Set the sensor configuration */
	rslt = bmi160_set_sens_conf(dev);
	
	return rslt;
}

#if defined(FIFO_POLL) || defined(FIFO_WM_INT)
int8_t Open_BMI160_FIFO(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	uint16_t index = 0;
	#if defined(FIFO_WM_INT)
	struct bmi160_int_settg int_config;
	#endif

	fifo_frame.data = fifo_buff;
	fifo_frame.length = 1024;
	dev->fifo = &fifo_frame;

	/* Clear FIFO configuration register */
	rslt = bmi160_set_fifo_config(BMI160_FIFO_CONFIG_1_MASK, BMI160_DISABLE, dev);
	if (rslt != BMI160_OK) 
	{
		PDEBUG("Clear FIFO configuration register error, error code: %d\r\n", rslt);
		return rslt;
	}
	
	//for 100 Hz ODR, with heading
#if defined(ACC_ONLY)
	rslt = bmi160_set_fifo_wm(43, dev);//7*50=350 bytes, water mark value was word unit, cann't be greater than 255, 87*4=348
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_wm error, error code: %d\r\n", rslt);
	}
	rslt = bmi160_set_fifo_config(BMI160_FIFO_ACCEL|BMI160_FIFO_HEADER, BMI160_ENABLE, dev);
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_config error, error code: %d\r\n", rslt);
	}
#elif defined(GYRO_ONLY)
	bmi160_set_fifo_wm(87, dev);	//7*50=350 bytes, water mark value was word unit, cann't be greater than 255, 87*4=348
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_wm error, error code: %d\r\n", rslt);
	}
	//rslt = bmi160_set_fifo_config(BMI160_FIFO_GYRO, BMI160_ENABLE, dev);
	rslt = bmi160_set_fifo_config(BMI160_FIFO_GYRO|BMI160_FIFO_HEADER, BMI160_ENABLE, dev);
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_config error, error code: %d\r\n", rslt);
	}
#elif defined(ACC_GYRO)
	//13*50=650 bytes, water mark value was word unit, cann't be greater than 255, 162*4=648
	//bmi160_set_fifo_wm(162, dev);	
	bmi160_set_fifo_wm(162, dev);	
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_wm error, error code: %d\r\n", rslt);
	}
	//rslt = bmi160_set_fifo_config(BMI160_FIFO_ACCEL|BMI160_FIFO_GYRO, BMI160_ENABLE, dev);
	rslt = bmi160_set_fifo_config(BMI160_FIFO_ACCEL |BMI160_FIFO_GYRO|BMI160_FIFO_HEADER, BMI160_ENABLE, dev);
	if(rslt != BMI160_OK)
	{
		PDEBUG("bmi160_set_fifo_config error, error code: %d\r\n", rslt);
	}
#endif

	/* Flush FIFO */
	rslt = bmi160_set_fifo_flush(dev);
	if (rslt != BMI160_OK) 
	{
	        PDEBUG("Flush FIFO error, error code: %d\r\n", rslt);
	        return rslt;
	}

#if defined(FIFO_WM_INT)
	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 2

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_GYRO_FIFO_WATERMARK_INT;// Choosing interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_DISABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// non-latched output
	int_config.fifo_wtm_int_en = BMI160_ENABLE;//Enable FIFO watermark interrupt

	/* Set the FIFO watermark interrupt */
	bmi160_set_int_config(&int_config, dev);
#endif
	
	return rslt;
}

int8_t Close_BMI160_FIFO(struct bma4_dev *dev)
{
	int8_t rslt = BMI160_OK;
	
	return rslt;
}
#endif

#if defined(STEP_COUNTER)
int8_t Open_BMI160_STEP_COUNTER(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	uint8_t step_enable = 1;//enable the step counter

	rslt = bmi160_set_step_counter(step_enable,  dev);

#if 0
	//struct bmp38x_acc_step_detect_int_cfg int_cfg;
	//PDEBUG("step_detector_en=%X, min_threshold=%X\r\n", int_cfg.step_detector_en, int_cfg.min_threshold);
	struct bmi160_int_settg int_cfg;
	int_cfg.int_type = BMI160_STEP_DETECT_INT;
	int_cfg.int_type_cfg.acc_step_detect_int.step_detector_en = BMI160_ENABLE;
	int_cfg.int_type_cfg.acc_step_detect_int.step_detector_mode = BMI160_STEP_DETECT_ROBUST;//BMI160_STEP_DETECT_NORMAL, BMI160_STEP_DETECT_SENSITIVE
	bmi160_set_int_config(&int_cfg, dev);
#endif	
	return rslt;
}

int8_t Close_BMI160_STEP_COUNTER(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#if defined(STEP_DETECTOR)
int8_t Open_BMI160_STEP_DETECTOR(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;
	
	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1
	
	/* Select the Interrupt type */
	int_config.int_type = BMI160_STEP_DETECT_INT;// Choosing Step Detector interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active High output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur =BMI160_LATCH_DUR_NONE;// non-latched output
	
	/* Select the Step Detector interrupt parameters, Kindly use the recommended settings for step detector */
	int_config.int_type_cfg.acc_step_detect_int.step_detector_mode = BMI160_STEP_DETECT_NORMAL;
	int_config.int_type_cfg.acc_step_detect_int.step_detector_en = BMI160_ENABLE;// 1-enable, 0-disable the step detector
	
	/* Set the Step Detector interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */

	return rslt;
}

int8_t Close_BMI160_STEP_DETECTOR(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;
	
	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_NONE;// Interrupt channel/pin 1
	
	/* Select the Interrupt type */
	int_config.int_type = BMI160_STEP_DETECT_INT;// Choosing Step Detector interrupt

	/* Set the Step Detector interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	
	return rslt;
}
#endif

#if defined(ANY_MOTION)
int8_t Open_BMI160_ANY_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_ANY_MOTION_INT;// Choosing Any motion interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_10_MILLI_SEC;// non-latched output

	/* Select the Any-motion interrupt parameters */
	int_config.int_type_cfg.acc_any_motion_int.anymotion_en = BMI160_ENABLE;// 1- Enable the any-motion, 0- disable any-motion 
	int_config.int_type_cfg.acc_any_motion_int.anymotion_x = BMI160_ENABLE;// Enabling x-axis for any motion interrupt
	int_config.int_type_cfg.acc_any_motion_int.anymotion_y = BMI160_ENABLE;// Enabling y-axis for any motion interrupt
	int_config.int_type_cfg.acc_any_motion_int.anymotion_z = BMI160_ENABLE;// Enabling z-axis for any motion interrupt
	int_config.int_type_cfg.acc_any_motion_int.anymotion_dur = 0;// any-motion duration
	int_config.int_type_cfg.acc_any_motion_int.anymotion_thr = 0x14;// (2-g range) -> (slope_thr) * 3.91 mg, (4-g range) -> (slope_thr) * 7.81 mg, (8-g range) ->(slope_thr) * 15.63 mg, (16-g range) -> (slope_thr) * 31.25 mg 

	/* Set the Any-motion interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev  */

	return rslt;
}

int8_t Close_BMI160_ANY_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_NONE;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_ANY_MOTION_INT;// Choosing Any motion interrupt

	/* Set the Any-motion interrupt */
	rslt = bmi160_set_int_config(&int_config, dev);
		
	return rslt;
}
#endif

#if defined(NO_MOTION)
int8_t Open_BMI160_NO_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	
	return rslt;
}

int8_t Close_BMI160_NO_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#if defined(SIG_MOTION)
int8_t Open_BMI160_SIG_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_SIG_MOTION_INT;// Choosing Any motion interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// non-latched output

	/* Select the Any-motion interrupt parameters */
	int_config.int_type_cfg.acc_sig_motion_int.sig_en = BMI160_ENABLE;// 1- Enable the sig-motion, 0- disable sig-motion 
	int_config.int_type_cfg.acc_sig_motion_int.sig_mot_skip = 1;//skip time for significant interrupt: 0=1.5s, 1=3s, 2=6s, 3=12s
	int_config.int_type_cfg.acc_sig_motion_int.sig_mot_proof = 1;//proof time for significant interrupt: 0=0.25s, 1=0.5s, 2=1s, 3=2s
	int_config.int_type_cfg.acc_sig_motion_int.sig_mot_thres = 0x14;//// (2-g range) -> (slope_thr) * 3.91 mg, (4-g range) -> (slope_thr) * 7.81 mg, (8-g range) ->(slope_thr) * 15.63 mg, (16-g range) -> (slope_thr) * 31.25 mg 
	int_config.int_type_cfg.acc_sig_motion_int.sig_data_src = BMI160_DISABLE;//data source 0- filter & 1 pre-filter

	/* Set the Any-motion interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev  */
		
	return rslt;
}

int8_t Close_BMI160_SIG_MOTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif


#if defined(TAP)
int8_t Open_BMI160_TAP(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;
#if 1

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_DOUBLE_TAP_INT;
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_DISABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_80_MILLI_SEC;// latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_tap_int.tap_en = BMI160_ENABLE;//tap enable, 1 - enable, 0 - disable 
	int_config.int_type_cfg.acc_tap_int.tap_data_src = BMI160_ENABLE;//data source 0- filter & 1 pre-filter
	/*tap duration, select the length of the time window for the second shock event for double tap detection, 
	000 - 50ms, 001 - 100ms, 010 - 150ms, 011 - 200ms, 100 - 250ms, 101 - 375ms, 110 - 500ms, 111 - 700ms*/
	int_config.int_type_cfg.acc_tap_int.tap_dur = 2;
	int_config.int_type_cfg.acc_tap_int.tap_quiet = BMI160_DISABLE;//tap quiet, 0 - 30ms, 1 - 20ms
	int_config.int_type_cfg.acc_tap_int.tap_shock = BMI160_DISABLE;//tap shock duration, 0 - 50ms, 1 - 75ms
	int_config.int_type_cfg.acc_tap_int.tap_thr = 2;//tap threshold

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */

	uint8_t data;
	data = 0x30;
	bmi160_set_regs(0x50, &data, 1, dev);

	data = 0x30;
	bmi160_set_regs(0x55, &data, 1, dev);
	#if 0
	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_2;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_DOUBLE_TAP_INT;// Choosing flat interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_DISABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_80_MILLI_SEC;// non-latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_tap_int.tap_en = BMI160_ENABLE;//tap enable, 1 - enable, 0 - disable 
	int_config.int_type_cfg.acc_tap_int.tap_data_src = BMI160_ENABLE;//data source 0- filter & 1 pre-filter
	/*tap duration, select the length of the time window for the second shock event for double tap detection, 
	000 - 50ms, 001 - 100ms, 010 - 150ms, 011 - 200ms, 100 - 250ms, 101 - 375ms, 110 - 500ms, 111 - 700ms*/
	int_config.int_type_cfg.acc_tap_int.tap_dur = 2;
	int_config.int_type_cfg.acc_tap_int.tap_quiet = BMI160_DISABLE;//tap quiet, 0 - 30ms, 1 - 20ms
	int_config.int_type_cfg.acc_tap_int.tap_shock = BMI160_DISABLE;//tap shock duration, 0 - 50ms, 1 - 75ms
	int_config.int_type_cfg.acc_tap_int.tap_thr = 2;//tap threshold

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	#endif
#else
/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_SINGLE_TAP_INT;// Choosing flat interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_DISABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_80_MILLI_SEC;// non-latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_tap_int.tap_en = BMI160_ENABLE;//tap enable, 1 - enable, 0 - disable 
	int_config.int_type_cfg.acc_tap_int.tap_data_src = BMI160_ENABLE;//data source 0- filter & 1 pre-filter
	/*tap duration, select the length of the time window for the second shock event for double tap detection, 
	000 - 50ms, 001 - 100ms, 010 - 150ms, 011 - 200ms, 100 - 250ms, 101 - 375ms, 110 - 500ms, 111 - 700ms*/
	int_config.int_type_cfg.acc_tap_int.tap_dur = 2;
	int_config.int_type_cfg.acc_tap_int.tap_quiet = BMI160_DISABLE;//tap quiet, 0 - 30ms, 1 - 20ms
	int_config.int_type_cfg.acc_tap_int.tap_shock = BMI160_DISABLE;//tap shock duration, 0 - 50ms, 1 - 75ms
	int_config.int_type_cfg.acc_tap_int.tap_thr = 2;//tap threshold

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	#if 0
	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_2;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_DOUBLE_TAP_INT;// Choosing flat interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// non-latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_tap_int.tap_en = BMI160_ENABLE;//tap enable, 1 - enable, 0 - disable 
	int_config.int_type_cfg.acc_tap_int.tap_data_src = BMI160_ENABLE;//data source 0- filter & 1 pre-filter
	/*tap duration, select the length of the time window for the second shock event for double tap detection, 
	000 - 50ms, 001 - 100ms, 010 - 150ms, 011 - 200ms, 100 - 250ms, 101 - 375ms, 110 - 500ms, 111 - 700ms*/
	int_config.int_type_cfg.acc_tap_int.tap_dur = 2;
	int_config.int_type_cfg.acc_tap_int.tap_quiet = BMI160_DISABLE;//tap quiet, 0 - 30ms, 1 - 20ms
	int_config.int_type_cfg.acc_tap_int.tap_shock = BMI160_DISABLE;//tap shock duration, 0 - 50ms, 1 - 75ms
	int_config.int_type_cfg.acc_tap_int.tap_thr = 2;//tap threshold

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	#endif
#endif
		
	return rslt;
}

int8_t Close_BMI160_TAP(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_NONE;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_DOUBLE_TAP_INT;// Choosing flat interrupt

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	
	return rslt;
}
#endif

#if defined(ORIENTATION)
int8_t Open_BMI160_ORIENTATION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;

	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_ORIENT_INT;// Choosing Any motion interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// non-latched output

	/* Select the orientation interrupt parameters */
	int_config.int_type_cfg.acc_orient_int.orient_mode = 3;//
	int_config.int_type_cfg.acc_orient_int.orient_blocking = 3;//
	int_config.int_type_cfg.acc_orient_int.orient_hyst = BMI160_ENABLE;//
	int_config.int_type_cfg.acc_orient_int.orient_theta = BMI160_ENABLE;//
	int_config.int_type_cfg.acc_orient_int.orient_ud_en = 1;//
	int_config.int_type_cfg.acc_orient_int.axes_ex = 1;//
	int_config.int_type_cfg.acc_orient_int.orient_en = BMI160_ENABLE;//
	
	/* Set the Any-motion interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev  */
		
	return rslt;
}

int8_t Close_BMI160_ORIENTATION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#if defined(FLAT_DETECTION)
int8_t Open_BMI160_FLAT_DETECTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_FLAT_INT;// Choosing flat interrupt
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// non-latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_flat_int.flat_en = BMI160_ENABLE;// 1-enable, 0-disable the flat interrupt
	int_config.int_type_cfg.acc_flat_int.flat_theta = 8;// threshold for detection of flat position in range from 0 to 44.8.
	int_config.int_type_cfg.acc_flat_int.flat_hy = 1;// Flat hysteresis
	int_config.int_type_cfg.acc_flat_int.flat_hold_time = 1;// Flat hold time (0 -> 0 ms, 1 -> 640 ms, 2 -> 1280 ms, 3 -> 2560 ms)

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	
	return rslt;
}

int8_t Close_BMI160_FLAT_DETECTION(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_NONE;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_FLAT_INT;// Choosing flat interrupt

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
		
	return rslt;
}
#endif

#if defined(LOW_G)
int8_t Open_BMI160_LOW_G(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_int_settg int_config;

	/* Select the Interrupt channel/pin */
	int_config.int_channel = BMI160_INT_CHANNEL_1;// Interrupt channel/pin 1

	/* Select the Interrupt type */
	int_config.int_type = BMI160_ACC_LOW_G_INT;
	/* Select the interrupt channel/pin settings */
	int_config.int_pin_settg.output_en = BMI160_ENABLE;// Enabling interrupt pins to act as output pin
	int_config.int_pin_settg.output_mode = BMI160_DISABLE;// Choosing push-pull mode for interrupt pin
	int_config.int_pin_settg.output_type = BMI160_ENABLE;// Choosing active high output
	int_config.int_pin_settg.edge_ctrl = BMI160_ENABLE;// Choosing edge triggered output
	int_config.int_pin_settg.input_en = BMI160_DISABLE;// Disabling interrupt pin to act as input
	int_config.int_pin_settg.latch_dur = BMI160_LATCH_DUR_NONE;// latched output

	/* Select the Flat interrupt parameters */
	int_config.int_type_cfg.acc_low_g_int.low_dur = 24;/*! low-g interrupt trigger delay */
	int_config.int_type_cfg.acc_low_g_int.low_thres = 12;/*! low-g interrupt trigger threshold */
	int_config.int_type_cfg.acc_low_g_int.low_hyst = BMI160_ENABLE;/*! hysteresis of low-g interrupt */
	int_config.int_type_cfg.acc_low_g_int.low_mode = BMI160_ENABLE;/*! 0 - single-axis mode ,1 - axis-summing mode */
	int_config.int_type_cfg.acc_low_g_int.low_data_src = BMI160_ENABLE; /*! data source 0- filter & 1 pre-filter */
	int_config.int_type_cfg.acc_low_g_int.low_en = BMI160_ENABLE; 	/*! 1 - enable low-g, 0 - disable low-g */

	/* Set the Flat interrupt */
	rslt = bmi160_set_int_config(&int_config, dev); /* sensor is an instance of the structure bmi160_dev */
	if(rslt == BMI160_OK)
	{
		PDEBUG("bmi160_set_int_config for low-g success\r\n");
	}
	else
	{
		PDEBUG("bmi160_set_int_config for low-g failed\r\n");
	}
	return rslt;
}

int8_t Close_BMI160_LOW_G(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#if defined(HIGH_G)
int8_t Open_BMI160_HIGH_G(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}

int8_t Close_BMI160_HIGH_G(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#if defined(FOC)
int8_t Start_BMI160_FOC(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	/* FOC configuration structure */
	struct bmi160_foc_conf foc_conf;
	/* Structure to store the offsets */
	struct bmi160_offsets offsets;
	
	/* Enable FOC for accel with target values of z = 1g ; x,y as 0g */
	foc_conf.acc_off_en = BMI160_ENABLE;
	foc_conf.foc_acc_x  = BMI160_FOC_ACCEL_0G;
	foc_conf.foc_acc_y  = BMI160_FOC_ACCEL_0G;
	foc_conf.foc_acc_z  = BMI160_FOC_ACCEL_POSITIVE_G;
	
	/* Enable FOC for gyro */
	foc_conf.foc_gyr_en = BMI160_ENABLE;
	foc_conf.gyro_off_en = BMI160_ENABLE;

	rslt = bmi160_start_foc(&foc_conf, &offsets, dev);
	
	//if (rslt == BMI160_OK) 
	{
		PDEBUG("FOC DONE SUCCESSFULLY\r\n");
		PDEBUG("OFFSET VALUES AFTER FOC : \r\n");
		PDEBUG("OFFSET VALUES ACCEL X : %d\r\n",offsets.off_acc_x);
		PDEBUG("OFFSET VALUES ACCEL Y : %d\r\n",offsets.off_acc_y);
		PDEBUG("OFFSET VALUES ACCEL Z : %d\r\n",offsets.off_acc_z);
		PDEBUG("OFFSET VALUES GYRO  X : %d\r\n",offsets.off_gyro_x);
		PDEBUG("OFFSET VALUES GYRO  Y : %d\r\n",offsets.off_gyro_y);
		PDEBUG("OFFSET VALUES GYRO  Z : %d\r\n",offsets.off_gyro_z);	
	}
	//else
	{
		PDEBUG("bmi160_start_foc failed\r\n");
	}
	
	/* After start of FOC offsets will be updated automatically and 
	 * the data will be very much close to the target values of measurement */
	return rslt;
}

int8_t Stop_BMI160_FOC(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
		
	return rslt;
}
#endif

#define GRAVITY_EARTH  (9.80665f)

static float lsb_to_mps2(int16_t val, float g_range, uint8_t bit_width)
{
    float half_scale = ((float)(1 << bit_width) / 2.0f);

    return (GRAVITY_EARTH * val * g_range) / half_scale;
}


static void calibration_bmi160(struct bmi160_dev *dev)
{
    int8_t rslt;
    int cnt = 0;

    struct bmi160_sensor_data acc_data[20], gyr_data[20];
    struct bmi160_offsets offset = {0};

#if 0
    rslt = bmi160_init(&bmi160dev);

    if (rslt == BMI160_OK)
    {
        printf("BMI160 initialization success !\n");
        printf("Chip ID 0x%X\n", bmi160dev.chip_id);
    }
    else
    {
        printf("BMI160 initialization failure !\n");
        exit(COINES_E_FAILURE);
    }
#endif
    /* Select the Output data rate, range of accelerometer sensor */
    dev->accel_cfg.odr = BMI160_ACCEL_ODR_100HZ;
    dev->accel_cfg.range = BMI160_ACCEL_RANGE_8G;
    dev->accel_cfg.bw = BMI160_ACCEL_BW_NORMAL_AVG4;

    /* Select the power mode of accelerometer sensor */
    dev->accel_cfg.power = BMI160_ACCEL_NORMAL_MODE;

    /* Select the Output data rate, range of Gyroscope sensor */
    dev->gyro_cfg.odr = BMI160_GYRO_ODR_100HZ;
    dev->gyro_cfg.range = BMI160_GYRO_RANGE_2000_DPS;
    dev->gyro_cfg.bw = BMI160_GYRO_BW_NORMAL_MODE;

    /* Select the power mode of Gyroscope sensor */
    dev->gyro_cfg.power = BMI160_GYRO_NORMAL_MODE;

    /* Set the sensor configuration */
    rslt = bmi160_set_sens_conf(dev);
	/* after sensor init introduce 200 msec sleep */
    dev->delay_ms(200);
	
    while (cnt < 20)
    {
        /* To read both Accel and Gyro data */
        bmi160_get_sensor_data((BMI160_ACCEL_SEL | BMI160_GYRO_SEL), &acc_data[cnt], &gyr_data[cnt], dev);

        PDEBUG("ax:%d\tay:%d\taz:%d\n", acc_data[cnt].x, acc_data[cnt].y, acc_data[cnt].z);
        PDEBUG("gx:%d\tgy:%d\tgz:%d\n", gyr_data[cnt].x, gyr_data[cnt].y, gyr_data[cnt].z);

        dev->delay_ms(10);
        cnt = cnt + 1;
    }
    
	/*! Average value calculation */
	double x_avg_mg=0, y_avg_mg=0, z_avg_mg=0;
	double x_avg_dps=0, y_avg_dps=0, z_avg_dps=0;

	/*! Average value calculation */
	double xval, yval, zval;
	for (int i=0; i<20; ++i) {
		/* ( 'datapoint' * '2G' * '2' * '1000mg') / ('scaling factor for 12 bits') */
		xval = lsb_to_mps2(acc_data[i].x, 8, 16);
		yval = lsb_to_mps2(acc_data[i].y, 8, 16);
		zval = lsb_to_mps2(acc_data[i].z, 8, 16);
		x_avg_mg += xval/20.0;
		y_avg_mg += yval/20.0;
		z_avg_mg += zval/20.0;
        /*
		xval = lsb_to_dps(gyr_data[i].x, 2000, 16);
		yval = lsb_to_dps(gyr_data[i].y, 2000, 16);
		zval = lsb_to_dps(gyr_data[i].z, 2000, 16);
		x_avg_dps += xval;
		y_avg_dps += yval;
		z_avg_dps += zval;
		*/
	}
	
	PDEBUG("\nPre-calibration acc offset: x=%4.1lfmg y=%4.1lfmg z=%4.1lfmg\n", x_avg_mg, y_avg_mg, z_avg_mg);
	PDEBUG("\nPre-calibration gyr offset: x=%4.1lfdps y=%4.1lfdps z=%4.1lfdps\n", x_avg_dps, y_avg_dps, z_avg_dps);
	
	memset(&offset, 0x00, sizeof(offset));
	rslt = bmi160_get_offsets(&offset, dev);
	PDEBUG("\nbmi160_start_foc() offset: x=%d y=%d z=%d\n", 
                        offset.off_acc_x, offset.off_acc_y, offset.off_acc_z);

    const struct bmi160_foc_conf acc_foc = {
        .foc_acc_x = BMI160_FOC_ACCEL_0G,
        .foc_acc_y = BMI160_FOC_ACCEL_0G,
        .foc_acc_z = BMI160_FOC_ACCEL_POSITIVE_G,
        .acc_off_en = BMI160_ENABLE,

        .foc_gyr_en = BMI160_ENABLE,
        .gyro_off_en = BMI160_ENABLE
    };
	rslt = bmi160_start_foc( &acc_foc, &offset, dev);
	PDEBUG("\nbmi160_start_foc() offset: x=%d y=%d z=%d\n", 
                        offset.off_acc_x, offset.off_acc_y, offset.off_acc_z);
	
	memset(&offset, 0x00, sizeof(offset));
	rslt = bmi160_get_offsets(&offset, dev);
	PDEBUG("\nbmi160_start_foc() offset: x=%d y=%d z=%d\n", 
                        offset.off_acc_x, offset.off_acc_y, offset.off_acc_z);
	if ( 0 != rslt ) PDEBUG("Unknown communication error. Exiting...\n");
}


int8_t Init_BMI160(struct bmi160_dev *dev)
{
	int8_t rslt = BMI160_OK;
	
#if defined(USE_I2C_INTERFACE)
	dev->intf = BMI160_I2C_INTF;
	dev->id = 0x69;
	dev->read = SensorAPI_I2Cx_Read;
	dev->write = SensorAPI_I2Cx_Write;
	dev->delay_ms = HAL_Delay;
	dev->read_write_len = 32;
#elif defined(USE_SPI_INTERFACE)
	dev->intf = BMI160_SPI_INTF;
	dev->id = 0;
	dev->read = SensorAPI_BMX160_SPIx_Read;
	dev->write = SensorAPI_BMX160_SPIx_Write;
	dev->delay_ms = HAL_Delay;
	dev->read_write_len = 32;
#endif
	
	rslt = bmi160_soft_reset(dev);
	dev->delay_ms(200);

	PDEBUG("BMI160 initialize started\r\n");
	
	rslt = bmi160_init(dev);
	if (rslt != BMI160_OK)
	{
		PDEBUG("bmi160_init() failed, error code: %d\r\n", rslt);
		return rslt;
	}
	else
	{
			PDEBUG("bmi160_init success, read chip ID: %02X\r\n", dev->chip_id);
	}

#if defined(ACC_SELFTEST)
	/* Call the "bmi160_init" API as a prerequisite before performing self test
		 * since invoking self-test will reset the sensor */

	rslt = bmi160_perform_self_test(BMI160_ACCEL_ONLY, dev);
	/* Utilize the enum BMI160_GYRO_ONLY instead of BMI160_ACCEL_ONLY
	   to perform self test for gyro */
	if (rslt == BMI160_OK) 
	{
		PDEBUG("ACCEL SELF TEST RESULT SUCCESS\r\n");
	} 
	else 
	{
		PDEBUG("ACCEL SELF TEST RESULT FAIL\r\n");
	}
	for(;;)
	{
		;
	}
#elif defined(GYRO_SELFTEST)
	/* Call the "bmi160_init" API as a prerequisite before performing self test
		 * since invoking self-test will reset the sensor */

	rslt = bmi160_perform_self_test(BMI160_GYRO_ONLY, dev);
	/* Utilize the enum BMI160_GYRO_ONLY instead of BMI160_ACCEL_ONLY
	   to perform self test for gyro */
	if (rslt == BMI160_OK) 
	{
		PDEBUG("GYRO SELF TEST RESULT SUCCESS\r\n");
	} 
	else 
	{
		PDEBUG("GYRO SELF TEST RESULT FAIL\r\n");
	}
	for(;;)
	{
		;
	}
#elif defined(ACC_GYRO_SELFTEST)
	rslt = bmi160_perform_self_test(BMI160_ACCEL_ONLY, dev);
	/* Utilize the enum BMI160_GYRO_ONLY instead of BMI160_ACCEL_ONLY
	   to perform self test for gyro */
	if (rslt == BMI160_OK) 
	{
		PDEBUG("ACCEL SELF TEST RESULT SUCCESS\r\n");
	} 
	else 
	{
		PDEBUG("ACCEL SELF TEST RESULT FAIL\r\n");
	}

	rslt = bmi160_soft_reset(dev);
	dev->delay_ms(200);

	PDEBUG("BMI160 initialize started\r\n");
	
	rslt = bmi160_init(dev);
	if (rslt != BMI160_OK)
	{
		PDEBUG("bmi160_init() failed, error code: %d\r\n", rslt);
		return rslt;
	}
	else
	{
			PDEBUG("bmi160_init success, read chip ID: %02X\r\n", dev->chip_id);
	}

	/* Call the "bmi160_init" API as a prerequisite before performing self test
		 * since invoking self-test will reset the sensor */

	rslt = bmi160_perform_self_test(BMI160_GYRO_ONLY, dev);
	/* Utilize the enum BMI160_GYRO_ONLY instead of BMI160_ACCEL_ONLY
	   to perform self test for gyro */
	if (rslt == BMI160_OK) 
	{
		PDEBUG("GYRO SELF TEST RESULT SUCCESS\r\n");
	} 
	else 
	{
		PDEBUG("GYRO SELF TEST RESULT FAIL\r\n");
	}
	for(;;)
	{
		;
	}
#endif
#if defined(FOC)
	calibration_bmi160(dev);
	#if 1
	dev->accel_cfg.odr = BMI160_ACCEL_ODR_100HZ;
	dev->accel_cfg.range = BMI160_ACCEL_RANGE_8G;
	/* Select the power mode of accelerometer sensor */
	dev->accel_cfg.power = BMI160_ACCEL_NORMAL_MODE;
	dev->accel_cfg.bw = BMI160_ACCEL_BW_NORMAL_AVG4;

	/* Select the Output data rate, range of Gyroscope sensor */
	dev->gyro_cfg.odr = BMI160_GYRO_ODR_100HZ;
	dev->gyro_cfg.range = BMI160_GYRO_RANGE_2000_DPS;
	/* Select the power mode of Gyroscope sensor */
	dev->gyro_cfg.power = BMI160_GYRO_NORMAL_MODE; 
	dev->gyro_cfg.bw = BMI160_GYRO_BW_NORMAL_MODE;

	/* Set the sensor configuration */
	rslt = bmi160_set_sens_conf(dev);
	#else
	Open_BMI160_ACC(dev);
	Open_BMI160_GYRO(dev);
	#endif

	dev->delay_ms(200);
	Start_BMI160_FOC(dev);
#endif

#if defined(ACC_ONLY)
	Open_BMI160_ACC(dev);
	//Close_BMI160_GYRO(dev);
	
#elif defined(GYRO_ONLY)
	Close_BMI160_ACC(dev);
	Open_BMI160_GYRO(dev);

	/* Select the power mode */
	dev->accel_cfg.power = BMI160_ACCEL_SUSPEND_MODE;
	dev->gyro_cfg.power = BMI160_GYRO_NORMAL_MODE; 

	/*  Set the Power mode  */
	rslt = bmi160_set_power_mode(dev);
#elif defined(ACC_GYRO)
	Open_BMI160_ACC(dev);
	Open_BMI160_GYRO(dev);
#endif

#if defined(STEP_COUNTER)
	Open_BMI160_STEP_COUNTER(dev);
#endif

#if defined(STEP_DETECTOR)
	Open_BMI160_STEP_DETECTOR(dev);
#endif

#if defined(TAP)
	Open_BMI160_TAP(dev);
#endif

#if defined(ORIENTATION)
Open_BMI160_ORIENTATION(dev);
#endif


#if defined(ANY_MOTION)
	Open_BMI160_ANY_MOTION(dev);
#endif

#if defined(NO_MOTION)
	Open_BMI160_NO_MOTION(dev);
#endif

#if defined(SIG_MOTION)
	Open_BMI160_SIG_MOTION(dev);
#endif

#if defined(FLAT_DETECTION)
	Open_BMI160_FLAT_DETECTION(dev);
#endif

#if defined(LOW_G)
	Open_BMI160_LOW_G(dev);
#endif

#if defined(HIGH_G)
	Open_BMI160_HIGH_G(dev);
#endif

#if defined(STEP_DETECTOR) || defined(TAP) || defined(ANY_MOTION) || defined(NO_MOTION) || defined(FLAT_DETECTION) || defined(ORIENTATION) || defined(HIGH_G) || defined(LOW_G)
	Enable_MCU_INT1_Pin();
#endif

#if defined(FIFO_POLL)
	Open_BMI160_FIFO(dev);
	//Enable_MCU_INT1_Pin();
#elif defined(FIFO_WM_INT)
	Open_BMI160_FIFO(dev);
	Enable_MCU_INT1_Pin();
#endif

#if 0
	/* Select the power mode */
	dev->accel_cfg.power = BMI160_ACCEL_LOWPOWER_MODE;
	dev->gyro_cfg.power = BMI160_GYRO_SUSPEND_MODE; 
	
	/*  Set the Power mode  */
	rslt = bmi160_set_power_mode(dev);
	PDEBUG("bmi160_set_power_mode 1 rslt=%d\r\n", rslt);
#endif
	
	return rslt;
}

#if defined(FIFO_POLL) || defined(STEP_COUNTER)
void BMI160_Timer_Callback(TimerHandle_t xTimer)
{
	bmi160_fifo_ready = 1;
}
#endif

#if defined(STEP_DETECTOR) || defined(TAP) || defined(ANY_MOTION) || defined(NO_MOTION) || defined(FLAT_DETECTION) || defined(ORIENTATION) || defined(HIGH_G) || defined(LOW_G)
void StartBMI160InterruptTask(void const * argument)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_dev *dev;
	volatile union bmi160_int_status int_status;

	dev = &bmi160dev;

	for(;;)
	{
		if(int1_flag == 1)
		{
			memset(int_status.data, 0x00, sizeof(int_status.data));
			bmi160_get_int_status(BMI160_INT_STATUS_ALL, &int_status, dev);
			PDEBUG("data: 0x%02X, 0x%02X, 0x%02X, 0x%02X\r\n", int_status.data[0], int_status.data[1], int_status.data[2], int_status.data[3]);

			if(int_status.bit.step)
			{
				PDEBUG("Step detector interrupt occured\r\n");
			}

			if(int_status.bit.anym)
			{
				PDEBUG("Any motion interrupt occured\r\n");
			}

			if(int_status.bit.nomo)
			{
				PDEBUG("No motion interrupt occured\r\n");
			}

			if(int_status.bit.flat_int)
			{
				PDEBUG("Flat detection interrupt occured\r\n");
			}

			if(int_status.bit.s_tap)
			{
				PDEBUG("Single TAP interrupt occured\r\n");
			}

			if(int_status.bit.d_tap)
			{
				PDEBUG("Double TAP interrupt occured\r\n");
			}

			if(int_status.bit.tap_sign)
			{
				PDEBUG("TAP sign 1\r\n");
			}
			else
			{
				PDEBUG("TAP sign 0\r\n");
			}

			if(int_status.bit.orient)
			{
				PDEBUG("Orientation interrupt occured\r\n");
			}

			if(int_status.bit.high_g)
			{
				PDEBUG("High-g interrupt occured\r\n");
			}

			if(int_status.bit.low_g)
			{
				PDEBUG("Low-g interrupt occured\r\n");
			}

			memset(int_status.data, 0x00, sizeof(int_status.data));
			int1_flag = 0;
		}
	}
}
#endif


void StartBMI160Task(void const * argument)
{
	int8_t rslt = BMI160_OK;
	struct bmi160_dev *dev;

	dev = &bmi160dev;

	Init_BMI160(dev);

	#if defined(DATA_POLL)
	for(;;)
	{	
		#if 0
		/* To read only Accel data */
		rslt = bmi160_get_sensor_data(BMI160_ACCEL_SEL, &accel, NULL, dev);

		/* To read only Gyro data */
		rslt = bmi160_get_sensor_data(BMI160_GYRO_SEL, NULL, &gyro, dev);
		#else
		/* To read both Accel and Gyro data */
		bmi160_get_sensor_data((BMI160_ACCEL_SEL | BMI160_GYRO_SEL), &accel, &gyro, dev);
		#endif
		
		PDEBUG("ACCEL X : %d , Y : %d , Z : %d\r\n", accel.x, accel.y, accel.z);
		PDEBUG("GYRO X : %d , Y : %d , Z : %d\r\n", gyro.x, gyro.y, gyro.z);
		vTaskDelay(1000);
	}
	#endif

	#if defined(STEP_DETECTOR) || defined(TAP) || defined(ANY_MOTION) || defined(NO_MOTION) || defined(FLAT_DETECTION) || defined(ORIENTATION) || defined(HIGH_G) || defined(LOW_G)
	osThreadId BMI160InterruptTaskHandle;
	osThreadDef(BMI160InterruptTask, StartBMI160InterruptTask, osPriorityNormal, 0, 256);
  	BMI160InterruptTaskHandle = osThreadCreate(osThread(BMI160InterruptTask), NULL);
	#endif

	#if defined(FIFO_POLL) || defined(STEP_COUNTER)
	BMI160_Timer_Handler = xTimerCreate("BMI160FIFOReadTimer", 1010, pdTRUE, (void *)1, BMI160_Timer_Callback);
	if(BMI160_Timer_Handler != NULL)
	{
		xTimerStart(BMI160_Timer_Handler, 0);
	}
	#endif
	uint8_t reg_data;
	for(;;)
	{	
		#if defined(FIFO_POLL) || defined(FIFO_WM_INT) || defined(STEP_COUNTER)
		if(bmi160_fifo_ready == 1)
		{
		#if 1
			#if defined(FIFO_POLL) || defined(FIFO_WM_INT)
			#if 1
			/* Select the power mode */
			dev->accel_cfg.power = BMI160_ACCEL_NORMAL_MODE;
			dev->gyro_cfg.power = BMI160_GYRO_SUSPEND_MODE; 

			/*  Set the Power mode  */
			rslt = bmi160_set_power_mode(dev);
			//PDEBUG("bmi160_set_power_mode 2 rslt=%d\r\n", rslt);
			#endif
			
			/*dev = &bmi160dev;
			fifo_frame.data = fifo_buff;
			fifo_frame.length = 1024;
			dev->fifo = &fifo_frame;
			acc_frames_req = 50;
			gyro_frames_req = 50;*/
			fifo_frame.length = 50*7;
			acc_frames_req = 50;
			gyro_frames_req = 50;

			rslt = bmi160_get_fifo_data(dev);
			print_bmi160_rslt(rslt);
			#if 0
			PDEBUG("Fifo length:%d\r\n", dev->fifo->length);
			if(rslt == BMI160_OK)
			{
				#if defined(ACC_ONLY) || defined(ACC_GYRO)
					
				/* Parse the FIFO data to extract accelerometer data from the FIFO buffer */
				bmi160_extract_accel(fifo_acc_data, &acc_frames_req, dev);
				PDEBUG("bmi160_extract_accel rslt=%d\r\n", rslt);
				if (rslt == BMI160_OK) 
				{
					PDEBUG("Parsed accelerometer data frames: %d\r\n", acc_frames_req);

					/* Print the parsed accelerometer data from the FIFO buffer */
					for(bmi160_fifo_idx = 0 ; bmi160_fifo_idx < acc_frames_req ; bmi160_fifo_idx++) 
					{
						//PDEBUG("ACCEL[%d] %d %t %d %t %d\r\n", bmi160_fifo_idx , fifo_acc_data[bmi160_fifo_idx].x, fifo_acc_data[bmi160_fifo_idx].y, fifo_acc_data[bmi160_fifo_idx].z);
					}
				}
				#endif
				#if defined(GYRO_ONLY) || defined(ACC_GYRO)
				
				/* Parse the FIFO data to extract gyroscope data from the FIFO buffer */
				rslt = bmi160_extract_gyro(fifo_gyro_data, &gyro_frames_req, dev);
				PDEBUG("bmi160_extract_gyro rslt=%d\r\n", rslt);
				if (rslt == BMI160_OK) 
				{
					PDEBUG("Parsed gyroscope data frames: %d\r\n", gyro_frames_req);

					/* Print the parsed accelerometer data from the FIFO buffer */
					for(bmi160_fifo_idx = 0 ; bmi160_fifo_idx < gyro_frames_req ; bmi160_fifo_idx++) 
					{
						//PDEBUG("GYRO[%d] %d %t %d %t %d\r\n", bmi160_fifo_idx , fifo_gyro_data[bmi160_fifo_idx].x, fifo_gyro_data[bmi160_fifo_idx].y, fifo_gyro_data[bmi160_fifo_idx].z);
					}
				}
				#endif
			}
			#endif
			#endif
			
			#if defined(STEP_COUNTER)
			bmi160_step_print++;
			if(bmi160_step_print == 4)
			{
				rslt = bmi160_read_step_counter(&bmi160_step_count,  dev);
				PDEBUG("The step counter output is %u\r\n", bmi160_step_count);
				bmi160_step_print = 0;
			}
			#endif
			#if 1
			dev->accel_cfg.power = BMI160_ACCEL_LOWPOWER_MODE;
			dev->gyro_cfg.power = BMI160_GYRO_SUSPEND_MODE; 
			/*  Set the Power mode  */
			rslt = bmi160_set_power_mode(dev);
			//PDEBUG("bmi160_set_power_mode 3 rslt=%d\r\n", rslt);
			#endif
			bmi160_fifo_ready = 0;
		#endif	
		}
		#endif

		/*rslt = bmi160_get_regs(0x1C, &reg_data, 1, dev);
		PDEBUG("reg_0x1C=%02X\r\n", reg_data);
		rslt = bmi160_get_regs(0x1E, &reg_data, 1, dev);
		PDEBUG("reg_0x1E=%02X\r\n", reg_data);
		vTaskDelay(2000);*/
	}
}

#endif
